#!/usr/bin/env python3
"""
WiFi Analyzer
Wireless network analysis and security assessment tool.
Parameters:
    - interface: Wireless interface to use
    - scan_time: Duration of scan in seconds
    - channel: WiFi channel to monitor
Requires: scapy package and admin/root privileges
"""
from scapy.all import *
import argparse
import logging
from datetime import datetime
import json
import threading
import queue
from collections import defaultdict
import time
import os
import signal
import subprocess

class WifiAnalyzer:
    def __init__(self, interface, scan_time=60, channel=None):
        self.interface = interface
        self.scan_time = scan_time
        self.channel = channel
        self.stop_event = threading.Event()
        self.networks = defaultdict(dict)
        self.clients = defaultdict(set)
        self.packets_queue = queue.Queue()
        self.setup_logging()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def enable_monitor_mode(self):
        """Enable monitor mode on wireless interface"""
        try:
            self.logger.info(f"Enabling monitor mode on {self.interface}")
            subprocess.run(['airmon-ng', 'check', 'kill'], check=True)
            subprocess.run(['airmon-ng', 'start', self.interface], check=True)
            return True
        except Exception as e:
            self.logger.error(f"Error enabling monitor mode: {str(e)}")
            return False

    def disable_monitor_mode(self):
        """Disable monitor mode on wireless interface"""
        try:
            subprocess.run(['airmon-ng', 'stop', self.interface], check=True)
            subprocess.run(['service', 'NetworkManager', 'start'], check=True)
            self.logger.info("Monitor mode disabled")
        except Exception as e:
            self.logger.error(f"Error disabling monitor mode: {str(e)}")

    def set_channel(self, channel):
        """Set wireless interface to specific channel"""
        try:
            if channel:
                subprocess.run(
                    ['iwconfig', self.interface, 'channel', str(channel)],
                    check=True
                )
                self.logger.info(f"Set channel to {channel}")
        except Exception as e:
            self.logger.error(f"Error setting channel: {str(e)}")

    def packet_handler(self, pkt):
        """Handle captured packets"""
        if self.stop_event.is_set():
            return

        try:
            # Beacon frames
            if pkt.haslayer(Dot11Beacon):
                bssid = pkt[Dot11].addr2
                ssid = pkt[Dot11Elt].info.decode('utf-8', errors='ignore')
                channel = int(ord(pkt[Dot11Elt:3].info))
                
                try:
                    rssi = -(256-ord(pkt.notdecoded[-4:-3]))
                except:
                    rssi = -100
                    
                encryption = []
                cap = pkt.sprintf("{Dot11Beacon:%Dot11Beacon.cap%}")
                
                if 'privacy' in cap:
                    encryption.append('WEP')
                
                # Check for WPA/WPA2
                p = pkt[Dot11Elt]
                while isinstance(p, Dot11Elt):
                    if p.ID == 48:  # RSN (WPA2)
                        encryption.append('WPA2')
                    elif p.ID == 221 and p.info.startswith(b'\x00P\xf2\x01\x01\x00'):
                        encryption.append('WPA')
                    p = p.payload
                
                if not encryption:
                    encryption = ['OPEN']

                self.networks[bssid].update({
                    'ssid': ssid,
                    'channel': channel,
                    'encryption': encryption,
                    'rssi': rssi,
                    'beacons': self.networks[bssid].get('beacons', 0) + 1,
                    'last_seen': datetime.now().isoformat()
                })

            # Probe requests
            elif pkt.haslayer(Dot11ProbeReq):
                if pkt.haslayer(Dot11):
                    client_mac = pkt[Dot11].addr2
                    if client_mac not in ['00:00:00:00:00:00', 'ff:ff:ff:ff:ff:ff']:
                        if pkt.haslayer(Dot11Elt) and pkt[Dot11Elt].info:
                            ssid = pkt[Dot11Elt].info.decode('utf-8', errors='ignore')
                            if ssid != '':
                                self.clients[client_mac].add(ssid)

            # Data packets
            elif pkt.haslayer(Dot11):
                ds = pkt.FCfield & 0x3
                if ds == 1:
                    bssid = pkt[Dot11].addr1
                    client = pkt[Dot11].addr2
                elif ds == 2:
                    bssid = pkt[Dot11].addr2
                    client = pkt[Dot11].addr1
                else:
                    return

                if client not in ['00:00:00:00:00:00', 'ff:ff:ff:ff:ff:ff']:
                    if bssid in self.networks:
                        self.networks[bssid]['clients'] = \
                            self.networks[bssid].get('clients', set())
                        self.networks[bssid]['clients'].add(client)

            self.packets_queue.put(pkt)

        except Exception as e:
            self.logger.error(f"Error handling packet: {str(e)}")

    def channel_hopper(self):
        """Hop through WiFi channels"""
        while not self.stop_event.is_set():
            try:
                for channel in range(1, 14):
                    if self.stop_event.is_set():
                        break
                    self.set_channel(channel)
                    time.sleep(0.5)
            except Exception as e:
                self.logger.error(f"Error in channel hopper: {str(e)}")
                time.sleep(0.5)

    def analyze_network_security(self):
        """Analyze security of discovered networks"""
        security_issues = []
        
        for bssid, network in self.networks.items():
            # Check for open networks
            if 'OPEN' in network.get('encryption', []):
                security_issues.append({
                    'bssid': bssid,
                    'ssid': network['ssid'],
                    'issue': 'Open Network',
                    'severity': 'High',
                    'recommendation': 'Enable encryption'
                })
            
            # Check for WEP
            if 'WEP' in network.get('encryption', []):
                security_issues.append({
                    'bssid': bssid,
                    'ssid': network['ssid'],
                    'issue': 'WEP Encryption',
                    'severity': 'High',
                    'recommendation': 'Upgrade to WPA2/WPA3'
                })
            
            # Check for hidden SSIDs
            if not network['ssid']:
                security_issues.append({
                    'bssid': bssid,
                    'ssid': 'Hidden',
                    'issue': 'Hidden SSID',
                    'severity': 'Low',
                    'recommendation': 'SSID hiding provides minimal security'
                })

        return security_issues

    def save_results(self, output_file):
        # Convert sets to lists for JSON serialization
        networks_json = {}
        for bssid, network in self.networks.items():
            network_copy = network.copy()
            if 'clients' in network_copy:
                network_copy['clients'] = list(network_copy['clients'])
            networks_json[bssid] = network_copy

        clients_json = {}
        for client, ssids in self.clients.items():
            clients_json[client] = list(ssids)

        results = {
            'timestamp': datetime.now().isoformat(),
            'interface': self.interface,
            'scan_duration': self.scan_time,
            'networks': networks_json,
            'clients': clients_json,
            'security_analysis': self.analyze_network_security()
        }

        try:
            with open(output_file, 'w') as f:
                json.dump(results, f, indent=4)
            self.logger.info(f"Results saved to {output_file}")
        except Exception as e:
            self.logger.error(f"Error saving results: {str(e)}")

    def print_summary(self):
        self.logger.info("\nWiFi Scan Summary:")
        self.logger.info(f"Networks discovered: {len(self.networks)}")
        self.logger.info(f"Clients discovered: {len(self.clients)}")
        
        # Network details
        self.logger.info("\nNetworks:")
        for bssid, network in self.networks.items():
            self.logger.info(
                f"\nBSSID: {bssid}\n"
                f"SSID: {network['ssid']}\n"
                f"Channel: {network['channel']}\n"
                f"Encryption: {', '.join(network['encryption'])}\n"
                f"Signal: {network['rssi']} dBm\n"
                f"Clients: {len(network.get('clients', set()))}"
            )

        # Security issues
        security_issues = self.analyze_network_security()
        if security_issues:
            self.logger.info("\nSecurity Issues:")
            for issue in security_issues:
                self.logger.info(
                    f"\nNetwork: {issue['ssid']} ({issue['bssid']})\n"
                    f"Issue: {issue['issue']}\n"
                    f"Severity: {issue['severity']}\n"
                    f"Recommendation: {issue['recommendation']}"
                )

    def start_scan(self, output_file=None):
        if not self.enable_monitor_mode():
            return

        try:
            self.logger.info("Starting WiFi scan...")
            
            # Start channel hopper if no specific channel
            if not self.channel:
                hopper = threading.Thread(target=self.channel_hopper)
                hopper.daemon = True
                hopper.start()
            else:
                self.set_channel(self.channel)

            # Start packet capture
            sniff(
                iface=self.interface,
                prn=self.packet_handler,
                timeout=self.scan_time,
                store=0
            )

        except KeyboardInterrupt:
            self.logger.info("Scan interrupted by user")
        except Exception as e:
            self.logger.error(f"Error during scan: {str(e)}")
        finally:
            self.stop_event.set()
            if output_file:
                self.save_results(output_file)
            self.print_summary()
            self.disable_monitor_mode()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="WiFi Analyzer")
    parser.add_argument("interface", help="Wireless interface to use")
    parser.add_argument("--scan-time", type=int, default=60,
                      help="Duration of scan in seconds")
    parser.add_argument("--channel", type=int,
                      help="WiFi channel to monitor")
    parser.add_argument("--output", help="Output file for scan results")
    args = parser.parse_args()

    analyzer = WifiAnalyzer(args.interface, args.scan_time, args.channel)
    analyzer.start_scan(args.output)
